# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Waldiez model."""

from .extra_requirements import get_models_extra_requirements
from .model import DEFAULT_BASE_URLS, MODEL_NEEDS_BASE_URL, WaldiezModel
from .model_data import (
    WaldiezModelAPIType,
    WaldiezModelAWS,
    WaldiezModelData,
    WaldiezModelPrice,
)

__all__ = [
    "get_models_extra_requirements",
    "DEFAULT_BASE_URLS",
    "MODEL_NEEDS_BASE_URL",
    "WaldiezModel",
    "WaldiezModelData",
    "WaldiezModelPrice",
    "WaldiezModelAPIType",
    "WaldiezModelAWS",
]
